﻿using Infragistics.Windows.DataPresenter;
using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Interactivity;
using System.Xml.Linq;

namespace ChiRT.UI.Common.Behaviours
{
    public class GridLayoutPersistBehaviour : Behavior<DataPresenterBase>
    {
        public static string LayoutPath = AppDomain.CurrentDomain.BaseDirectory;
        
        private string panelName;
        public string PanelName
        {
            get { return panelName; }
            set
            {
                if (panelName != value && !string.IsNullOrEmpty(value))
                {
                    panelName = value;
                    fileName = string.Format("Grid.{0}.xml", String.Join(".", PanelName));
                    if (AssociatedObject != null)
                        SaveLayout(FileName);
                }
            }
        }

        public static string GetProduct(DependencyObject obj)
        {
            return (string)obj.GetValue(ProductProperty);
        }

        public static void SetProduct(DependencyObject obj, string value)
        {
            obj.SetValue(ProductProperty, value);
        }

        public static readonly DependencyProperty ProductProperty = DependencyProperty.RegisterAttached("Product", typeof(string), typeof(GridLayoutPersistBehaviour), new PropertyMetadata(string.Empty, OnProductChanged));

        private static void OnProductChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var behaviour = (GridLayoutPersistBehaviour)d;

            var newFileName = string.Format("{1}Grid.{0}.xml", String.Join(".", behaviour.PanelName.Split(System.IO.Path.GetInvalidFileNameChars())), e.NewValue);

            if (!string.IsNullOrEmpty((string)e.NewValue) && newFileName != behaviour.FileName)
            {
                behaviour.FileName = newFileName;
               // behaviour.LoadLayout(newFileName);
            }
        }

        private string fileName;
        protected string FileName
        {
            get { return fileName; }
            set
            {
                if (fileName != value)
                {
                    fileName = value;
                    LoadLayout(fileName);
                }
            }
        }

        protected override void OnAttached()
        {
            base.OnAttached();

            AssociatedObject.FieldPositionChanged += (o, e) =>
            {
                SaveLayout(FileName);
            };

            AssociatedObject.RecordFilterChanged += (o, e) =>
            {
                SaveLayout(FileName);
            };

        }

        protected  void LoadLayout(string fileName)
        {
            try
            {
                var path = System.IO.Path.Combine(LayoutPath, fileName);
                if (System.IO.File.Exists(path))
                    using (var file = System.IO.File.OpenRead(path))
                    {
                        AssociatedObject.LoadCustomizations(file);
                    }

            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error loading layout {0}", ex.Message);
            }
        }

        protected void SaveLayout(string fileName)
        {
            try
            {
                Application.Current.Dispatcher.BeginInvoke(new Action(() =>
                {
                    if (AssociatedObject.IsLoaded && AssociatedObject.IsMeasureValid)
                    {
                        //AssociatedObject.SaveLayout(SystemInfo.LayoutDir, fileName);
                        var path = System.IO.Path.Combine(LayoutPath, fileName);
                        var tmpFileName = System.IO.Path.ChangeExtension(path, ".tmp");
                        var bakFileName = System.IO.Path.ChangeExtension(path, ".bak");

                        System.IO.File.WriteAllText(tmpFileName, RemoveAdditionalLayouts(AssociatedObject));
                        ReplaceWithRetry(tmpFileName, path, bakFileName);
                    }
                }));
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error saving layout {0}", ex.Message);
            }
        }

        private static string RemoveAdditionalLayouts(DataPresenterBase grid)
        {
            string customizations = grid.SaveCustomizations();
            var layoutXml = XElement.Parse(customizations);
            if (layoutXml.Descendants("fieldLayout").Count() > 1)
                layoutXml.Descendants("fieldLayouts").Elements("fieldLayout").First().ElementsAfterSelf().Remove();

            return layoutXml.ToString();
        }


        public static void ReplaceWithRetry(string sourceFileName, string destFileName, string bakFileName)
        {
            try
            {
                if (File.Exists(destFileName))
                    System.IO.File.Replace(sourceFileName, destFileName, bakFileName, true);
                else
                    System.IO.File.Copy(sourceFileName, destFileName);
            }
            catch (System.IO.IOException)
            {
                // happens sometimes on saving layout, perhaps due to antivirus scanning
                System.Threading.Thread.Sleep(100);
                if (File.Exists(destFileName))
                    System.IO.File.Replace(sourceFileName, destFileName, bakFileName, true);
                else
                    System.IO.File.Copy(sourceFileName, destFileName);
            }
        }

    }
}
